@@ -62,13 +62,15 @@ BEGIN_EVENT_TABLE( DISPLAY_FOOTPRINTS_FRAME, PCB_BASE_FRAME )
     EVT_CLOSE( DISPLAY_FOOTPRINTS_FRAME::OnCloseWindow )
     EVT_CHOICE( ID_ON_ZOOM_SELECT, DISPLAY_FOOTPRINTS_FRAME::OnSelectZoom )
     EVT_CHOICE( ID_ON_GRID_SELECT, DISPLAY_FOOTPRINTS_FRAME::OnSelectGrid )
+    EVT_MENU( ID_GRID_SETTINGS, DISPLAY_FOOTPRINTS_FRAME::OnGridSettings )
 END_EVENT_TABLE()
 
 
 DISPLAY_FOOTPRINTS_FRAME::DISPLAY_FOOTPRINTS_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
-    PCB_BASE_FRAME( aKiway, aParent, FRAME_CVPCB_DISPLAY, _( "Footprint Viewer" ),
-                    wxDefaultPosition, wxDefaultSize,
-                    KICAD_DEFAULT_DRAWFRAME_STYLE, FOOTPRINTVIEWER_FRAME_NAME )
+        PCB_BASE_FRAME( aKiway, aParent, FRAME_CVPCB_DISPLAY, _( "Footprint Viewer" ),
+                        wxDefaultPosition, wxDefaultSize,
+                        KICAD_DEFAULT_DRAWFRAME_STYLE, FOOTPRINTVIEWER_FRAME_NAME ),
+        m_currentComp( nullptr )
 {
     // Give an icon
     wxIcon  icon;
@@ -131,13 +133,13 @@ DISPLAY_FOOTPRINTS_FRAME::DISPLAY_FOOTPRINTS_FRAME( KIWAY* aKiway, wxWindow* aPa
     m_auimgr.SetManagedWindow( this );
 
     CreateInfoBar();
-    m_auimgr.AddPane( m_mainToolBar, EDA_PANE().HToolbar().Name( "MainToolbar" )
+    m_auimgr.AddPane( m_mainToolBar, EDA_PANE().HToolbar().Name( wxT( "MainToolbar" ) )
                       .Top().Layer( 6 ) );
-    m_auimgr.AddPane( m_optionsToolBar, EDA_PANE().VToolbar().Name( "OptToolbar" )
+    m_auimgr.AddPane( m_optionsToolBar, EDA_PANE().VToolbar().Name( wxT( "OptToolbar" ) )
                       .Left().Layer( 3 ) );
-    m_auimgr.AddPane( GetCanvas(), EDA_PANE().Canvas().Name( "DrawFrame" )
+    m_auimgr.AddPane( GetCanvas(), EDA_PANE().Canvas().Name( wxT( "DrawFrame" ) )
                       .Center() );
-    m_auimgr.AddPane( m_messagePanel, EDA_PANE().Messages().Name( "MsgPanel" )
+    m_auimgr.AddPane( m_messagePanel, EDA_PANE().Messages().Name( wxT( "MsgPanel" ) )
                       .Bottom().Layer( 6 ) );
 
     FinishAUIInitialization();
@@ -451,21 +453,20 @@ FOOTPRINT* DISPLAY_FOOTPRINTS_FRAME::GetFootprint( const wxString& aFootprintNam
 void DISPLAY_FOOTPRINTS_FRAME::InitDisplay()
 {
     CVPCB_MAINFRAME*      parentframe = (CVPCB_MAINFRAME *) GetParent();
+    COMPONENT*            comp = parentframe->GetSelectedComponent();
     FOOTPRINT*            footprint = nullptr;
     const FOOTPRINT_INFO* fpInfo = nullptr;
 
-    GetBoard()->DeleteAllFootprints();
-    GetCanvas()->GetView()->Clear();
-
     wxString footprintName = parentframe->GetSelectedFootprint();
 
-    if( footprintName.IsEmpty() )
-    {
-        COMPONENT* comp = parentframe->GetSelectedComponent();
+    if( footprintName.IsEmpty() && comp )
+        footprintName = comp->GetFPID().Format();
 
-        if( comp )
-            footprintName = comp->GetFPID().GetUniStringLibId();
-    }
+    if( m_currentFootprint == footprintName && m_currentComp == comp )
+        return;
+
+    GetBoard()->DeleteAllFootprints();
+    GetCanvas()->GetView()->Clear();
 
     INFOBAR_REPORTER infoReporter( m_infoBar );
     m_infoBar->Dismiss();
@@ -480,7 +481,11 @@ void DISPLAY_FOOTPRINTS_FRAME::InitDisplay()
     }
 
     if( footprint )
+    {
         GetBoard()->Add( footprint );
+        m_currentFootprint = footprintName;
+        m_currentComp = comp;
+    }
 
     if( fpInfo )
         SetStatusText( wxString::Format( _( "Lib: %s" ), fpInfo->GetLibNickname() ), 0 );
