@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -95,21 +95,22 @@ static LIB_SYMBOL* dummy()
 }
 
 
-SCH_SYMBOL::SCH_SYMBOL( const wxPoint& aPos, SCH_ITEM* aParent ) :
+SCH_SYMBOL::SCH_SYMBOL( const wxPoint& aPosition, EDA_ITEM* aParent ) :
     SCH_ITEM( aParent, SCH_SYMBOL_T )
 {
-    Init( aPos );
+    Init( aPosition );
 }
 
 
 SCH_SYMBOL::SCH_SYMBOL( const LIB_SYMBOL& aSymbol, const LIB_ID& aLibId,
-                        const SCH_SHEET_PATH* aSheet, int unit, int convert, const wxPoint& pos ) :
-    SCH_ITEM( nullptr, SCH_SYMBOL_T )
+                        const SCH_SHEET_PATH* aSheet, int aUnit, int aConvert,
+                        const wxPoint& aPosition, EDA_ITEM* aParent ) :
+    SCH_ITEM( aParent, SCH_SYMBOL_T )
 {
-    Init( pos );
+    Init( aPosition );
 
-    m_unit      = unit;
-    m_convert   = convert;
+    m_unit      = aUnit;
+    m_convert   = aConvert;
     m_lib_id    = aLibId;
 
     std::unique_ptr< LIB_SYMBOL > part;
@@ -146,8 +147,8 @@ SCH_SYMBOL::SCH_SYMBOL( const LIB_SYMBOL& aSymbol, const LIB_ID& aLibId,
 
 
 SCH_SYMBOL::SCH_SYMBOL( const LIB_SYMBOL& aSymbol, const SCH_SHEET_PATH* aSheet,
-                        const PICKED_SYMBOL& aSel, const wxPoint& pos ) :
-    SCH_SYMBOL( aSymbol, aSel.LibId, aSheet, aSel.Unit, aSel.Convert, pos )
+                        const PICKED_SYMBOL& aSel, const wxPoint& aPosition, EDA_ITEM* aParent ) :
+    SCH_SYMBOL( aSymbol, aSel.LibId, aSheet, aSel.Unit, aSel.Convert, aPosition, aParent )
 {
     // Set any fields that were modified as part of the symbol selection
     for( const std::pair<int, wxString>& i : aSel.Fields )
@@ -226,6 +227,15 @@ EDA_ITEM* SCH_SYMBOL::Clone() const
 }
 
 
+bool SCH_SYMBOL::IsMissingLibSymbol() const
+{
+    if( !m_part )
+        return true;
+
+    return false;
+}
+
+
 void SCH_SYMBOL::ViewGetLayers( int aLayers[], int& aCount ) const
 {
     aCount      = 4;
@@ -325,10 +335,12 @@ void SCH_SYMBOL::UpdatePins()
 
     unsigned i = 0;
 
-    for( LIB_PIN* libPin = m_part->GetNextPin(); libPin; libPin = m_part->GetNextPin( libPin ) )
-    {
-        wxASSERT( libPin->Type() == LIB_PIN_T );
+    std::vector<LIB_PIN*> pins = m_part->GetAllLibPins();
 
+    for( LIB_PIN* libPin : pins )
+    {
+        // NW: Don't filter by unit: this data-structure is used for all instances,
+        // some if which might have different units.
         if( libPin->GetConvert() && m_convert && ( m_convert != libPin->GetConvert() ) )
             continue;
 
@@ -399,6 +411,19 @@ int SCH_SYMBOL::GetUnitCount() const
 }
 
 
+
+void SCH_SYMBOL::PrintBackground( const RENDER_SETTINGS* aSettings, const wxPoint& aOffset )
+{
+    LIB_SYMBOL_OPTIONS opts;
+    opts.transform = m_transform;
+    opts.draw_visible_fields = false;
+    opts.draw_hidden_fields = false;
+
+    if( m_part )
+        m_part->PrintBackground( aSettings, m_pos + aOffset, m_unit, m_convert, opts );
+}
+
+
 void SCH_SYMBOL::Print( const RENDER_SETTINGS* aSettings, const wxPoint& aOffset )
 {
     LIB_SYMBOL_OPTIONS opts;
@@ -504,27 +529,29 @@ bool SCH_SYMBOL::IsReferenceStringValid( const wxString& aReferenceString )
 void SCH_SYMBOL::SetRef( const SCH_SHEET_PATH* sheet, const wxString& ref )
 {
     KIID_PATH path = sheet->Path();
-    bool      notInArray = true;
+    bool      found = false;
 
     // check to see if it is already there before inserting it
     for( SYMBOL_INSTANCE_REFERENCE& instance : m_instanceReferences )
     {
         if( instance.m_Path == path )
         {
+            found = true;
             instance.m_Reference = ref;
-            notInArray = false;
+            break;
         }
     }
 
-    if( notInArray )
-        AddHierarchicalReference( path, ref, m_unit );
+    if( !found )
+        AddHierarchicalReference( path, ref, m_unit, GetField( VALUE_FIELD )->GetText(),
+                                  GetField( FOOTPRINT_FIELD )->GetText() );
 
     for( std::unique_ptr<SCH_PIN>& pin : m_pins )
         pin->ClearDefaultNetName( sheet );
 
-    SCH_FIELD* rf = GetField( REFERENCE_FIELD );
-
-    rf->SetText( ref );  // for drawing.
+    // Schematic() is null for new components just added
+    if( !Schematic() || *sheet == Schematic()->CurrentSheet() )
+        m_fields[ REFERENCE_FIELD ].SetText( ref );
 
     // Reinit the m_prefix member if needed
     m_prefix = UTIL::GetRefDesPrefix( ref );
@@ -634,20 +661,28 @@ void SCH_SYMBOL::SetValue( const SCH_SHEET_PATH* sheet, const wxString& aValue )
     }
 
     KIID_PATH path = sheet->Path();
+    bool      found = false;
 
     // check to see if it is already there before inserting it
     for( SYMBOL_INSTANCE_REFERENCE& instance : m_instanceReferences )
     {
         if( instance.m_Path == path )
         {
+            found = true;
             instance.m_Value = aValue;
-            return;
+            break;
         }
     }
 
     // didn't find it; better add it
-    AddHierarchicalReference( path, UTIL::GetRefDesUnannotated( m_prefix ), m_unit,
-                              aValue, wxEmptyString );
+    if( !found )
+    {
+        AddHierarchicalReference( path, UTIL::GetRefDesUnannotated( m_prefix ), m_unit, aValue,
+                                  wxEmptyString );
+    }
+
+    if( Schematic() && *sheet == Schematic()->CurrentSheet() )
+        m_fields[ VALUE_FIELD ].SetText( aValue );
 }
 
 
@@ -685,20 +720,28 @@ void SCH_SYMBOL::SetFootprint( const SCH_SHEET_PATH* sheet, const wxString& aFoo
     }
 
     KIID_PATH path = sheet->Path();
+    bool      found = false;
 
     // check to see if it is already there before inserting it
     for( SYMBOL_INSTANCE_REFERENCE& instance : m_instanceReferences )
     {
         if( instance.m_Path == path )
         {
+            found = true;
             instance.m_Footprint = aFootprint;
-            return;
+            break;
         }
     }
 
     // didn't find it; better add it
-    AddHierarchicalReference( path, UTIL::GetRefDesUnannotated( m_prefix ), m_unit,
-                              wxEmptyString, aFootprint );
+    if( !found )
+    {
+        AddHierarchicalReference( path, UTIL::GetRefDesUnannotated( m_prefix ), m_unit,
+                                  wxEmptyString, aFootprint );
+    }
+
+    if( Schematic() && *sheet == Schematic()->CurrentSheet() )
+        m_fields[ FOOTPRINT_FIELD ].SetText( aFootprint );
 }
 
 
@@ -1912,6 +1955,35 @@ void SCH_SYMBOL::Plot( PLOTTER* aPlotter ) const
 }
 
 
+void SCH_SYMBOL::PlotPins( PLOTTER* aPlotter ) const
+{
+    if( m_part )
+    {
+        LIB_PINS  libPins;
+        m_part->GetPins( libPins, GetUnit(), GetConvert() );
+
+        // Copy the source to stay const
+        LIB_SYMBOL tempSymbol( *m_part );
+        LIB_PINS tempPins;
+        tempSymbol.GetPins( tempPins, GetUnit(), GetConvert() );
+
+        TRANSFORM transform = GetTransform();
+
+        // Copy the pin info from the symbol to the temp pins
+        for( unsigned i = 0; i < tempPins.size(); ++ i )
+        {
+            SCH_PIN* symbolPin = GetPin( libPins[ i ] );
+            LIB_PIN* tempPin = tempPins[ i ];
+
+            tempPin->SetName( symbolPin->GetShownName() );
+            tempPin->SetType( symbolPin->GetType() );
+            tempPin->SetShape( symbolPin->GetShape() );
+            tempPin->Plot( aPlotter, m_pos, false, transform);
+        }
+    }
+}
+
+
 bool SCH_SYMBOL::HasBrightenedPins()
 {
     for( const auto& pin : m_pins )
